@@ -13,6 +13,7 @@ class Issue
     @changes = []
     @board = board
 
+    raise "No board for issue #{key}" if board.nil?
     return unless @raw['changelog']
 
     load_history_into_changes
@@ -137,7 +138,15 @@ class Issue
     status = board.possible_statuses.find_by_name(name)
     return status if status
 
-    raise "Status name #{name.inspect} for issue #{key} not found in #{board.possible_statuses.collect(&:name).inspect}"
+    @board.project_config.file_system.log(
+      "Warning: Status name #{name.inspect} for issue #{key} not found in" \
+      " #{board.possible_statuses.collect(&:name).inspect}" \
+      "\n  See https://jirametrics.org/faq/#q1\n",
+      also_write_to_stderr: true
+    )
+    status = Status.new(name: name, category_name: 'In Progress')
+    board.possible_statuses << status
+    status
   end
 
   def first_status_change_after_created
@@ -251,6 +260,7 @@ class Issue
 
     blocked_link_texts = settings['blocked_link_text']
     stalled_threshold = settings['stalled_threshold_days']
+    flagged_means_blocked = !!settings['flagged_means_blocked'] # rubocop:disable Style/DoubleNegation
 
     blocking_issue_keys = []
 
@@ -273,7 +283,7 @@ class Issue
         blocking_stalled_changes: result
       )
 
-      if change.flagged?
+      if change.flagged? && flagged_means_blocked
         flag = change.value
         flag = nil if change.value == ''
       elsif change.status?
@@ -362,6 +372,45 @@ class Issue
     inserted_stalled
   end
 
+  # return [number of active seconds, total seconds] that this issue had up to the end_time.
+  # It does not include data before issue start or after issue end
+  def flow_efficiency_numbers end_time:, settings: @board.project_config.settings
+    issue_start, issue_stop = @board.cycletime.started_stopped_times(self)
+    return [0.0, 0.0] if !issue_start || issue_start > end_time
+
+    value_add_time = 0.0
+    end_time = issue_stop if issue_stop && issue_stop < end_time
+
+    active_start = nil
+    blocked_stalled_changes(end_time: end_time, settings: settings).each_with_index do |change, index|
+      break if change.time > end_time
+
+      if index.zero?
+        active_start = change.time if change.active?
+        next
+      end
+
+      # Already active and we just got another active.
+      next if active_start && change.active?
+
+      if change.active?
+        active_start = change.time
+      elsif active_start && change.time >= issue_start
+        # Not active now but we have been. Record the active time.
+        change_delta = change.time - [issue_start, active_start].max
+        value_add_time += change_delta
+        active_start = nil
+      end
+    end
+
+    if active_start
+      change_delta = end_time - [issue_start, active_start].max
+      value_add_time += change_delta if change_delta.positive?
+    end
+
+    [value_add_time, end_time - issue_start]
+  end
+
   def all_subtask_activity_times
     subtask_activity_times = []
     @subtasks.each do |subtask|
@@ -493,6 +542,20 @@ class Issue
     comparison
   end
 
+  def discard_changes_before cutoff_time
+    rejected_any = false
+    @changes.reject! do |change|
+      reject = change.status? && change.time <= cutoff_time && change.artificial? == false
+      if reject
+        (@discarded_changes ||= []) << change
+        rejected_any = true
+      end
+      reject
+    end
+
+    (@discarded_change_times ||= []) << cutoff_time if rejected_any
+  end
+
   def dump
     result = +''
     result << "#{key} (#{type}): #{compact_text summary, 200}\n"
@@ -500,21 +563,59 @@ class Issue
     assignee = raw['fields']['assignee']
     result << "  [assignee] #{assignee['name'].inspect} <#{assignee['emailAddress']}>\n" unless assignee.nil?
 
-    raw['fields']['issuelinks'].each do |link|
+    raw['fields']['issuelinks']&.each do |link|
       result << "  [link] #{link['type']['outward']} #{link['outwardIssue']['key']}\n" if link['outwardIssue']
       result << "  [link] #{link['type']['inward']} #{link['inwardIssue']['key']}\n" if link['inwardIssue']
     end
-    changes.each do |change|
+    history = [] # time, type, detail
+
+    started_at, stopped_at = board.cycletime.started_stopped_times(self)
+    history << [started_at, nil, '↓↓↓↓ Started here ↓↓↓↓', true] if started_at
+    history << [stopped_at, nil, '↑↑↑↑ Finished here ↑↑↑↑', true] if stopped_at
+
+    @discarded_change_times&.each do |time|
+      history << [time, nil, '↑↑↑↑ Changes discarded ↑↑↑↑', true]
+    end
+
+    (changes + (@discarded_changes || [])).each do |change|
       value = change.value
       old_value = change.old_value
 
-      message = "  [change] #{change.time.strftime '%Y-%m-%d %H:%M:%S %z'} [#{change.field}] "
+      message = +''
       message << "#{compact_text(old_value).inspect} -> " unless old_value.nil? || old_value.empty?
       message << compact_text(value).inspect
-      message << " (#{change.author})"
-      message << ' <<artificial entry>>' if change.artificial?
-      result << message << "\n"
+      if change.artificial?
+        message << ' (Artificial entry)' if change.artificial?
+      else
+        message << " (Author: #{change.author})"
+      end
+      history << [change.time, change.field, message, change.artificial?]
     end
+
+    result << "  History:\n"
+    type_width = history.collect { |_time, type, _detail, _artificial| type&.length || 0 }.max
+    history.sort! do |a, b|
+      if a[0] == b[0]
+        if a[1].nil?
+          1
+        elsif b[1].nil?
+          -1
+        else
+          a[1] <=> b[1]
+        end
+      else
+        a[0] <=> b[0]
+      end
+    end
+    history.each do |time, type, detail, _artificial|
+      if type.nil?
+        type = '-' * type_width
+      else
+        type = (' ' * (type_width - type.length)) << type
+      end
+      result << "    #{time.strftime '%Y-%m-%d %H:%M:%S %z'} [#{type}] #{detail}\n"
+    end
+
     result
   end
 
